@@ -38,6 +38,7 @@ static void test_nwrap_passwd_group(void **state)
 /* Test libnsl */
 static void test_nwrap_hostent(void **state)
 {
+#ifdef HAVE_NONNULL_GETHOSTENT
 	struct hostent *he;
 
 	(void) state; /* unused */
@@ -48,6 +49,13 @@ static void test_nwrap_hostent(void **state)
 	assert_non_null(he);
 
 	endhostent();
+#else
+	(void) state; /* unused */
+
+	sethostent(0);
+	gethostent();
+	endhostent();
+#endif
 }
 
 static void test_nwrap_gethostname(void **state)
@@ -67,7 +75,10 @@ static void test_nwrap_getaddrinfo_local(void **state)
 {
 	struct addrinfo hints;
 	struct addrinfo *res;
-	struct sockaddr_in *sinp;
+	union {
+		struct sockaddr *sa;
+		struct sockaddr_in *in;
+	} addr;
 	int rc;
 
 	(void) state; /* unused */
@@ -84,11 +95,15 @@ static void test_nwrap_getaddrinfo_local(void **state)
 	assert_int_equal(res->ai_family, AF_INET);
 	assert_int_equal(res->ai_socktype, SOCK_STREAM);
 
+#ifdef HAVE_GETADDRINFO_SETS_CANONNAME_FOR_IPADDRESSES
+	assert_string_equal(res->ai_canonname, "127.0.0.1");
+#else /* HAVE_GETADDRINFO_SETS_CANONNAME_FOR_IPADDRESSES */
 	assert_null(res->ai_canonname);
+#endif /* HAVE_GETADDRINFO_SETS_CANONNAME_FOR_IPADDRESSES */
 
-	sinp = (struct sockaddr_in *)res->ai_addr;
+	addr.sa = res->ai_addr;
 
-	assert_int_equal(ntohl(sinp->sin_addr.s_addr), INADDR_LOOPBACK);
+	assert_int_equal(ntohl(addr.in->sin_addr.s_addr), INADDR_LOOPBACK);
 
 	freeaddrinfo(res);
 }
